@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2004 Michael Terry
+ * Copyright (c) 2004-2005 Michael Terry
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 
 #include "../config.h"
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #include "fio.h" /* for fio_get_info_from_file */
 #include "help.h"
@@ -41,21 +42,6 @@
 #include "xpad-session-manager.h"
 #include "xpad-tray.h"
 
-#include "../images/sticky.xpm"
-
-#if defined (G_OS_UNIX)
-  /* required by mkdir */
- #include <sys/stat.h>
- #include <sys/types.h>
-#elif defined (G_OS_WIN32)
- /* required by CreateDirectory */
- #include <Winbase.h>
-#elif defined (G_OS_BEOS)
- /* required by CreateDirectory */
- #include <be/storage/Directory.h>
-#endif
-
-
 /* Seems that some systems (sun-sparc-solaris2.8 at least), need the following three #defines. 
    These were provided by Alan Mizrahi <alan@cesma.usb.ve>.
 */
@@ -74,7 +60,12 @@
 
 static gint xpad_argc;
 static gchar **xpad_argv;
-static gboolean make_new_pad_if_none;
+static gboolean option_hide_old;
+static gboolean option_nonew;
+static gboolean option_new;
+static gboolean option_version;
+static gboolean option_quit;
+static gchar *option_smid;
 static gchar *config_dir;
 static gchar *program_path;
 static gchar *server_filename;
@@ -83,19 +74,23 @@ static FILE *output;
 static gboolean xpad_translucent = FALSE;
 static XpadPadGroup *pad_group;
 
-static gint      process_args               (gint argc, gchar **argv, gboolean local_args);
-static gint      xpad_app_check_if_others   (void);
+static gboolean  process_local_args         (gint *argc, gchar **argv[]);
+static gboolean  process_remote_args        (gint *argc, gchar **argv[], gboolean have_gtk);
 
 static gboolean  config_dir_exists          (void);
 static gchar    *make_config_dir            (void);
 static void      register_stock_icons       (void);
-static void      set_default_icon           (void);
 static gint      xpad_app_load_pads         (void);
+static void      xpad_app_quit_if_no_pads   (XpadPadGroup *group);
+static gboolean  xpad_app_pass_args         (void);
+static gboolean  xpad_app_open_proc_file    (void);
 
 
 static void
 xpad_app_init (int argc, char **argv)
 {
+	gboolean first_time;
+	gboolean have_gtk;
 /*	GdkVisual *visual;*/
 	
 	/* Set up i18n */
@@ -106,11 +101,32 @@ xpad_app_init (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 	
-	gtk_init (&argc, &argv);
+	have_gtk = gtk_init_check (&argc, &argv);
 	xpad_argc = argc;
 	xpad_argv = argv;
 	output = stdout;
 	
+	/* Set up config directory. */
+	first_time = !config_dir_exists ();
+	config_dir = make_config_dir ();
+	
+	/* create master socket name */
+	server_filename = g_build_filename (xpad_app_get_config_dir (), "server", NULL);
+	
+	if (!have_gtk)
+	{
+		/* We don't have GTK+, but we can still do
+		   --version or --help and such.  Plus, we
+		   can pass commands to a remote instance. */
+		process_local_args (&xpad_argc, &xpad_argv);
+		if (!xpad_app_pass_args ())
+		{
+			process_remote_args (&xpad_argc, &xpad_argv, FALSE);
+			fprintf (output, "%s\n", _("Xpad is a graphical program.  Please run it from your desktop."));
+		}
+		exit (0);
+	}
+	
 	g_set_application_name (_("Xpad"));
 	gdk_set_program_class (PACKAGE);
 	
@@ -130,37 +146,38 @@ xpad_app_init (int argc, char **argv)
 	else
 		program_path = NULL;
 	
-	/* Set up config directory. */
-	if (!config_dir_exists ())
-	{
-		show_help ();
-	}
-	config_dir = make_config_dir ();
+	process_local_args (&xpad_argc, &xpad_argv);
 	
-	make_new_pad_if_none = TRUE;
-	process_args (xpad_argc, xpad_argv, TRUE);
-	
-	if (xpad_app_check_if_others ())
+	if (xpad_app_pass_args ())
 		exit (0);
 	
+	/* Race condition here, between calls */
+	xpad_app_open_proc_file ();
+	
 	register_stock_icons ();
-	set_default_icon ();
+	gtk_window_set_default_icon_name (PACKAGE);
+	
+	process_remote_args (&xpad_argc, &xpad_argv, TRUE);
 	
 	xpad_tray_open ();
 	xpad_session_manager_init ();
 	
 	/* load all pads */
 	pad_group = NULL;
-	if (xpad_app_load_pads () == 0) {
-		if (make_new_pad_if_none) {
+	if (xpad_app_load_pads () == 0 && !option_new) {
+		if (!option_nonew) {
 			GtkWidget *pad = xpad_pad_new (pad_group);
 			gtk_widget_show (pad);
 		}
-		else if (!xpad_tray_is_open ())
-			exit (0);
 	}
 	
-	process_args (xpad_argc, xpad_argv, FALSE);
+	xpad_app_quit_if_no_pads (pad_group);
+	
+	if (first_time)
+		show_help ();
+	
+	g_free (server_filename);
+	server_filename = NULL;
 }
 
 
@@ -232,34 +249,44 @@ config_dir_exists (void)
 	gchar *dir = NULL;
 	gboolean exists = FALSE;
 	
-#if defined (G_OS_UNIX)
-	
-	/* create a hidden directory under the user's home */
-	dir = g_build_filename (g_get_home_dir (), ".xpad", NULL);
-	
+	dir = g_build_filename (g_get_user_config_dir (), PACKAGE, NULL);
 	exists = g_file_test (dir, G_FILE_TEST_EXISTS);
-	
-#elif defined (G_OS_WIN32)
-	
-	/* If someone has a better place to put our stuff, I'm all ears. */
-	dir = g_build_filename (g_get_home_dir (), "xpad", NULL);
-	
-	exists = g_file_test (dir, G_FILE_TEST_EXISTS);
-	
-#elif defined (G_OS_BEOS)
-	
-	/* If someone has a better place to put our stuff, I'm all ears. */
-	dir = g_build_filename (g_get_home_dir (), "xpad", NULL);
-	
-	exists = g_file_test (dir, G_FILE_TEST_EXISTS);
-	
-#endif
-	
 	g_free (dir);
 	
+	if (!exists)
+	{
+		/* For backwards-compatibility, we see if the old location for
+		   configuration files exists.  It will be moved in make_config_dir */
+		dir = g_build_filename (g_get_home_dir (), "." PACKAGE, NULL);
+		exists = g_file_test (dir, G_FILE_TEST_EXISTS);
+		g_free (dir);
+	}
+	
 	return exists;
 }
 
+static void
+make_path (const gchar *path)
+{
+	GSList *dirs = NULL, *i;
+	gchar *dirname;
+	
+	dirname = g_strdup (path);
+	while (!g_file_test (dirname, G_FILE_TEST_EXISTS))
+	{
+		dirs = g_slist_prepend (dirs, dirname);
+		dirname = g_path_get_dirname (dirname);
+	}
+	g_free (dirname);
+	
+	for (i = dirs; i; i = i->next)
+	{
+		g_mkdir ((gchar *) i->data, 0700);
+		g_free (i->data);
+	}
+	g_slist_free (dirs);
+}
+
 /**
  * Creates the directory if it does not exist.
  * Returns newly allocated dir name, NULL if an error occurred.
@@ -269,31 +296,25 @@ make_config_dir (void)
 {
 	gchar *dir = NULL;
 	
-#if defined (G_OS_UNIX)
+	make_path (g_get_user_config_dir ());
 	
-	/* create a hidden directory under the user's home */
-	dir = g_build_filename (g_get_home_dir (), ".xpad", NULL);
+	dir = g_build_filename (g_get_user_config_dir (), PACKAGE, NULL);
 	
-	/* make sure directory exists */
-	mkdir (dir, 0700); /* give user all rights */
-	
-#elif defined (G_OS_WIN32)
-	
-	/* If someone has a better place to put our stuff, I'm all ears. */
-	dir = g_build_filename (g_get_home_dir (), "xpad", NULL);
-	
-	/* make sure directory exists */
-	CreateDirectory (dir, NULL); /* default security rights */
-	
-#elif defined (G_OS_BEOS)
-	
-	/* If someone has a better place to put our stuff, I'm all ears. */
-	dir = g_build_filename (g_get_home_dir (), "xpad", NULL);
-	
-	/* make sure directory exists */
-	CreateDirectory (dir, NULL);
-	
-#endif
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS))
+	{
+		gchar *olddir;
+		
+		/* For backwards-compatibility, we see if the old location for
+		   configuration files exists.  If so, we move it. */
+		olddir = g_build_filename (g_get_home_dir (), "." PACKAGE, NULL);
+		
+		if (g_file_test (olddir, G_FILE_TEST_EXISTS))
+			g_rename (olddir, dir);
+		else
+			g_mkdir (dir, 0700); /* give user all rights */
+		
+		g_free (olddir);
+	}
 	
 	return dir;
 }
@@ -349,89 +370,18 @@ xpad_app_alert_new (GtkWindow *parent, const gchar *stock,
 static void
 register_stock_icons (void)
 {
-	GtkIconSource *source;
-	GtkIconSet *set;
-	GtkIconFactory *factory;
 	GtkIconTheme *theme;
-	GdkPixbuf *pixbuf;
 	gchar *icon_dir;
 	
 	theme = gtk_icon_theme_get_default ();
 	icon_dir = g_build_filename (DATADIR, "icons", NULL);
 	gtk_icon_theme_prepend_search_path (theme, icon_dir);
 	g_free (icon_dir);
-	
-	factory = gtk_icon_factory_new ();
-	
-	pixbuf = gdk_pixbuf_new_from_xpm_data (sticky_xpm);
-	set = gtk_icon_set_new_from_pixbuf (pixbuf);
-	gtk_icon_factory_add (factory, "xpad-sticky", set);
-	gtk_icon_set_unref (set);
-	g_object_unref (pixbuf);
-	
-	source = gtk_icon_source_new ();
-	gtk_icon_source_set_icon_name (source, "xpad");
-	set = gtk_icon_set_new ();
-	gtk_icon_set_add_source (set, source);
-	gtk_icon_factory_add (factory, "xpad", set);
-	gtk_icon_set_unref (set);
-	gtk_icon_source_free (source);
-	
-	gtk_icon_factory_add_default (factory);
-	g_object_unref (factory);
 }
 
 
 static void
-set_default_icon (void)
-{
-	GList *icons = NULL;
-	GtkIconTheme *theme = gtk_icon_theme_get_default ();
-	GdkPixbuf *pixbuf;
-	
-	pixbuf = gtk_icon_theme_load_icon (theme,
-	                                   PACKAGE,
-	                                   16,
-	                                   0,
-	                                   NULL);
-	if (pixbuf)
-		icons = g_list_append (icons, pixbuf);
-	
-	pixbuf = gtk_icon_theme_load_icon (theme,
-	                                   PACKAGE,
-	                                   24,
-	                                   0,
-	                                   NULL);
-	if (pixbuf)
-		icons = g_list_append (icons, pixbuf);
-	
-	pixbuf = gtk_icon_theme_load_icon (theme,
-	                                   PACKAGE,
-	                                   32,
-	                                   0,
-	                                   NULL);
-	if (pixbuf)
-		icons = g_list_append (icons, pixbuf);
-	
-	pixbuf = gtk_icon_theme_load_icon (theme,
-	                                   PACKAGE,
-	                                   48,
-	                                   0,
-	                                   NULL);
-	if (pixbuf)
-		icons = g_list_append (icons, pixbuf);
-	
-	if (icons) {
-		gtk_window_set_default_icon_list (icons);
-		
-		g_list_foreach (icons, (GFunc) g_object_unref, NULL);
-		g_list_free (icons);
-	}
-}
-
-
-static void
-xpad_app_pad_hidden (XpadPadGroup *group, XpadPad *pad)
+xpad_app_quit_if_no_pads (XpadPadGroup *group)
 {
 	if (!xpad_tray_is_open ())
 	{
@@ -443,7 +393,12 @@ xpad_app_pad_hidden (XpadPadGroup *group, XpadPad *pad)
 				break;
 		}
 		if (!i)
-			gtk_main_quit ();
+		{
+			if (gtk_main_level () > 0)
+				gtk_main_quit ();
+			else
+				exit (0);
+		}
 		g_slist_free (list);
 	}
 }
@@ -451,7 +406,8 @@ xpad_app_pad_hidden (XpadPadGroup *group, XpadPad *pad)
 static void
 xpad_app_pad_added (XpadPadGroup *group, XpadPad *pad)
 {
-	g_signal_connect_swapped (pad, "hide", G_CALLBACK (xpad_app_pad_hidden), group);
+	g_signal_connect_swapped (pad, "closed", G_CALLBACK (xpad_app_quit_if_no_pads), group);
+	g_signal_connect_swapped (pad, "destroy", G_CALLBACK (xpad_app_quit_if_no_pads), group);
 }
 
 
@@ -492,7 +448,8 @@ xpad_app_load_pads (void)
 		    name[strlen (name) - 1] != '~')
 		{
 			GtkWidget *pad = xpad_pad_new_with_info (pad_group, name);
-			gtk_widget_show (pad);
+			if (!option_hide_old)
+				gtk_widget_show (pad);
 			
 			opened ++;
 		}
@@ -590,7 +547,6 @@ string_to_args (const char *string, char ***argv)
 }
 
 #include <errno.h>
-static gint xpad_app_open_proc_file (void);
 /* This reads a line from the proc file.  This line will contain arguments to process. */
 static void
 xpad_app_read_from_proc_file (void)
@@ -631,7 +587,7 @@ xpad_app_read_from_proc_file (void)
 	/* here we redirect singleton->priv->output to the socket */
 	output = fdopen (client_fd, "w");
 	
-	if (!process_args (argc, argv, FALSE))
+	if (!process_remote_args (&argc, &argv, TRUE))
 	{
 		/* if there were no non-local arguments, insert --new as argument */
 		gint c = 2;
@@ -639,7 +595,7 @@ xpad_app_read_from_proc_file (void)
 		v[0] = PACKAGE;
 		v[1] = "--new";
 		
-		process_args (c, v, FALSE);
+		process_remote_args (&c, &v, TRUE);
 		
 		g_free (v);
 	}
@@ -664,75 +620,37 @@ can_read_from_server_fd (GIOChannel *source, GIOCondition condition, gpointer da
 	return TRUE;
 }
 
-static gint
+static gboolean
 xpad_app_open_proc_file (void)
 {
 	GIOChannel *channel;
 	struct sockaddr_un master;
 	
-	unlink (server_filename);
+	g_unlink (server_filename);
 	
 	/* create the socket */
 	server_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
 	bzero (&master, sizeof (master)); 
 	master.sun_family = AF_LOCAL;
 	strcpy (master.sun_path, server_filename);
+	
 	if (bind (server_fd, (struct sockaddr *) &master, SUN_LEN (&master)))
-	{
-		return 1;
-	}
+		return FALSE;
 	
 	/* listen for connections */
-	listen (server_fd, 5);
+	if (listen (server_fd, 5))
+		return FALSE;
 	
 	/* set up input loop, waiting for read */
 	channel = g_io_channel_unix_new (server_fd);
 	g_io_add_watch (channel, G_IO_IN, can_read_from_server_fd, NULL);
 	g_io_channel_unref (channel);
 	
-	return 0;
+	return TRUE;
 }
 
 
-static void
-clipboard_get (GtkClipboard *clipboard, GtkSelectionData 
-	*selection_data, guint info, gpointer data)
-{
-	static gboolean first_time = TRUE;
-	
-	if (first_time)
-	{
-		first_time = FALSE;
-		
-		xpad_app_open_proc_file ();
-	}
-	
-	switch (info)
-	{
-	case 1:
-		/* Fill the selection with the filename of our proc_file.  On a 1, which is a 
-		   'are you alive?' ping, respond.  The other client will see this data and leave; we take
-		   over his arguments. */
-		gtk_selection_data_set (selection_data, 
-			gdk_atom_intern ("_XPAD_EXISTS", FALSE),
-			8,
-			(const guchar *) "",
-			0);
-	default:
-		break;
-	}
-}
-
-
-static void
-clipboard_clear (GtkClipboard *clipboard, gpointer data)
-{
-	/* No data needs to be freed.  This shouldn't be called anyway -- we retain
-	  control over clipboard throughout our life. */
-}
-
-
-static void
+static gboolean
 xpad_app_pass_args (void)
 {
 	int client_fd;
@@ -742,6 +660,7 @@ xpad_app_pass_args (void)
 	gchar *args = NULL;
 	gint size;
 	gint bytesRead;
+	gboolean connected = FALSE;
 	
 	/* create master socket */
 	client_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
@@ -751,6 +670,7 @@ xpad_app_pass_args (void)
 	/* connect to master socket */
 	if (connect (client_fd, (struct sockaddr *) &master, SUN_LEN (&master)))
 		goto done;
+	connected = TRUE;
 	
 	size = args_to_string (xpad_argc, xpad_argv, &args) + 1;
 	
@@ -788,40 +708,8 @@ done:
 	close (client_fd);
 	
 	g_free (args);
-	g_free (server_filename);
-}
-
-
-static gint
-xpad_app_check_if_others (void)
-{
-	GtkClipboard *clipboard = gtk_clipboard_get (gdk_atom_intern ("_XPAD_EXISTS", FALSE));
-	GtkSelectionData *temp;
 	
-	/* create master socket name */
-	server_filename = g_build_filename (xpad_app_get_config_dir (), "server", NULL);
-	
-	if ((temp = gtk_clipboard_wait_for_contents (clipboard, 
-		gdk_atom_intern ("STRING", FALSE))))
-	{
-		xpad_app_pass_args ();
-		
-		gtk_selection_data_free (temp);
-		
-		return 1;
-	}
-	else
-	{
-		/* set up target list with simple string target w/ value of 1 */
-		GtkTargetEntry targets[] = {{"STRING", 0, 1}};
-		
-		/* no one else is alive.  claim the clipboard. */
-		
-		gtk_clipboard_set_with_data (clipboard, targets, 1, 
-			clipboard_get, clipboard_clear, NULL);
-		
-		return 0;
-	}
+	return connected;
 }
 
 
@@ -843,260 +731,113 @@ xpad_app_check_if_others (void)
 
 
 
-
 
 
 /**
  * Here are the functions called when arguments are passed to us.
  */
 
-
-enum {
-	XPAD_ARG_TYPE_NONE,
-	XPAD_ARG_TYPE_INT,
-	XPAD_ARG_TYPE_STRING
+static GOptionEntry local_options[] =
+{
+	{"version", 'v', 0, G_OPTION_ARG_NONE, &option_version, N_("Show version number and quit"), NULL},
+	{"hide-old", 'H', 0, G_OPTION_ARG_NONE, &option_hide_old, N_("Hide existing pads on startup"), NULL},
+	{"no-new", 'N', 0, G_OPTION_ARG_NONE, &option_nonew, N_("Don't create a new pad on startup if no previous pads exist"), NULL},
+	{NULL}
 };
 
-struct argument
+static GOptionEntry remote_options[] =
 {
-	gboolean local;
-	const gchar *short_name;
-	const gchar *long_name;
-	guint type;
-	union {
-		void (*func) (void);
-		void (*func_arg) (void *);
-	} callbacks;
-	const gchar *help_comment;
+	{"new", 'n', 0, G_OPTION_ARG_NONE, &option_new, N_("Create a new pad on startup even if pads already exist"), NULL},
+	{"quit", 'q', 0, G_OPTION_ARG_NONE, &option_quit, N_("Close all pads"), NULL},
+	{"sm-client-id", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &option_smid, NULL, NULL},
+	{NULL}
 };
 
-static void missing_companion_arg (const char argname[]);
-static void xpad_app_args_print_help (void);
-static void xpad_app_args_print_version (void);
-static void xpad_app_args_set_nonew (void);
-static void xpad_app_args_set_session (const gchar *str);
-static void xpad_app_args_spawn_pad (void);
-
-static const struct argument arguments[] =
+static gboolean
+process_local_args (gint *argc, gchar **argv[])
 {
-	{TRUE, "-h", "--help", XPAD_ARG_TYPE_NONE, {G_CALLBACK (xpad_app_args_print_help)}, N_("Prints usage information and exits")},
-	{TRUE, "-v", "--version", XPAD_ARG_TYPE_NONE, {G_CALLBACK (xpad_app_args_print_version)}, N_("Prints xpad version and exits")},
-	{TRUE, NULL, "--nonew", XPAD_ARG_TYPE_NONE, {G_CALLBACK (xpad_app_args_set_nonew)}, N_("Prevents xpad from creating a new pad on startup if no previous pads exist")},
+	GError *error = NULL;
+	GOptionContext *context;
+	gint argc_copy;
+	gchar **argv_copy;
 	
-	/* above are local arguments, the ones below are run by the remote instance */
+	option_version = FALSE;
+	option_nonew = FALSE;
+	option_hide_old = FALSE;
 	
-	/* --nonew is here twice because it does things for both local and remote instances.  in remote, it stops the remote instance from thinking there were no args and spawning a new pad */
-	{FALSE, NULL, "--nonew", XPAD_ARG_TYPE_NONE, {G_CALLBACK (xpad_app_args_set_nonew)}, NULL},
-	{FALSE, "-n", "--new", XPAD_ARG_TYPE_NONE, {G_CALLBACK (xpad_app_args_spawn_pad)}, N_("Causes xpad to create a new pad on startup even if pads already exist")},
-	{FALSE, "-q", "--quit", XPAD_ARG_TYPE_NONE, {G_CALLBACK (gtk_main_quit)}, N_("Causes all running xpad instances to close")},
-	{FALSE, NULL, "--sm-client-id", XPAD_ARG_TYPE_STRING, {G_CALLBACK (xpad_app_args_set_session)}, NULL}
-};
-
-
-static void
-xpad_app_args_spawn_pad (void)
-{
-	GtkWidget *pad = xpad_pad_new (pad_group);
-	gtk_widget_show (pad);
-}
-
-
-static void
-xpad_app_args_print_version (void)
-{
-	fprintf (output, _("Xpad %s\n"), PACKAGE_VERSION);
-	exit (0);
-}
-
-
-static void
-xpad_app_args_set_nonew (void)
-{
-	make_new_pad_if_none = FALSE;
-}
-
-
-static void
-xpad_app_args_set_session (const gchar *str)
-{
-	xpad_session_manager_set_id (str);
-}
-
-
-static void
-xpad_app_args_print_help (void)
-{
-	gchar *msg, *tmp_msg, *lmsg;
-	gint  largest_size, i;
-
-	for (largest_size = -1, i = 0; i < G_N_ELEMENTS (arguments); i++)
+	/* We make copies of argc and argv because we actually don't want the 
+	   behavior of g_option_context_parse() that removes entries from the
+	   array. */
+	argc_copy = *argc;
+	argv_copy = g_strdupv (*argv);
+	
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, local_options, GETTEXT_PACKAGE);
+	/* We do remote here as well, because we want --help to pick them up.  It
+	   can't hurt since they only set the global values that we reset later. */
+	g_option_context_add_main_entries (context, remote_options, GETTEXT_PACKAGE);
+	if (g_option_context_parse (context, &argc_copy, &argv_copy, &error))
 	{
-		if (arguments[i].help_comment)
+		if (option_version)
 		{
-			gchar *addition = g_strconcat ("  * ", 
-				arguments[i].short_name ? arguments[i].short_name : "",
-				arguments[i].type == XPAD_ARG_TYPE_INT ? " #" : "",
-				(arguments[i].short_name && arguments[i].long_name) ? ", " : "",
-				arguments[i].long_name ? arguments[i].long_name : "",
-				arguments[i].type == XPAD_ARG_TYPE_INT ? "=#" : "",
-				NULL);
-			if (largest_size == -1 || strlen (addition) > largest_size)
-			{
-				largest_size = strlen (addition);
-			}
-			g_free (addition);
+			fprintf (output, _("Xpad %s"), PACKAGE_VERSION);
+			fprintf (output, "\n");
+			exit (0);
 		}
 	}
-
-	msg = g_strdup_printf (_("Usage: %s [OPTIONS]\n\n"), g_get_prgname ());
-
-	for (i = 0; i < G_N_ELEMENTS (arguments); i++)
+	else
 	{
-		if (arguments[i].help_comment)
+		fprintf (output, "%s\n", error->message);
+		exit (1);
+	}
+	
+	g_option_context_free (context);
+	g_strfreev(argv_copy);
+	
+	return(option_version || option_nonew || option_hide_old);
+}
+
+static gboolean
+process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk)
+{
+	GError *error = NULL;
+	GOptionContext *context;
+	
+	option_new = FALSE;
+	option_quit = FALSE;
+	option_smid = NULL;
+	
+	context = g_option_context_new (NULL);
+	g_option_context_set_ignore_unknown_options (context, TRUE);
+	g_option_context_set_help_enabled (context, FALSE);
+	g_option_context_add_main_entries (context, remote_options, GETTEXT_PACKAGE);
+	if (g_option_context_parse (context, argc, argv, &error))
+	{
+		if (have_gtk && option_smid)
+			xpad_session_manager_set_id (option_smid);
+		
+		if (have_gtk && option_new)
 		{
-			gchar *padding;
-			gchar *addition = g_strconcat ("  * ", 
-				arguments[i].short_name ? arguments[i].short_name : "",
-				arguments[i].type == XPAD_ARG_TYPE_INT ? " #" : "",
-				(arguments[i].short_name && arguments[i].long_name) ? ", " : "",
-				arguments[i].long_name ? arguments[i].long_name : "",
-				arguments[i].type == XPAD_ARG_TYPE_INT ? "=#" : "",
-				NULL);
-
-			padding = g_strnfill (largest_size - strlen (addition) + 3, ' ');
-
-			tmp_msg = msg;
-			msg = g_strconcat (msg, addition, padding, _(arguments[i].help_comment), "\n", NULL);
-			g_free (tmp_msg);
+			GtkWidget *pad = xpad_pad_new (pad_group);
+			gtk_widget_show (pad);
+		}
+		
+		if (option_quit)
+		{
+			if (have_gtk && gtk_main_level () > 0)
+				gtk_main_quit ();
+			else
+				exit (0);
 		}
 	}
-	
-	lmsg = g_locale_from_utf8 (msg, -1, NULL, NULL, NULL);
-	
-	if (lmsg)
-		fprintf (output, lmsg);
-	
-	g_free (msg);
-	g_free (lmsg);
-	
-	exit (0);
-}
-
-
-static void
-missing_companion_arg (const char argname[])
-{
-	g_printerr (_("Missing companion argument to %s\n"), argname);
-	exit(1);
-}
-
-
-static gint
-process_args (gint argc, gchar **argv, gboolean local)
-{
-	gint i, j, recognized_at;
-	gint rv = 0;
-	size_t short_arglen[G_N_ELEMENTS (arguments)];
-	size_t long_arglen[G_N_ELEMENTS (arguments)];
-	
-	/* Set up array of argument lengths to avoid having to compute them every 
-	 * time through our inner loop.  This probably ought to be global, but it
-	 * won't matter all that much.
-	 */
-	for (j = G_N_ELEMENTS (arguments) - 1; j >= 0; j--) {
-		short_arglen[j] = arguments[j].short_name ? strlen (arguments[j].short_name) : 0;
-		long_arglen[j] = arguments[j].long_name ? strlen (arguments[j].long_name) : 0;
+	else
+	{
+		fprintf (output, "%s\n", error->message);
+		/* Don't quit.  Bad options passed to the main xpad program by other
+		   iterations shouldn't close the main one. */
 	}
 	
-	for (i = 1; i < argc; i++) {
-		gboolean longform;
-		
-		/* Find matching argument; there can be only one.
-		 */
-		recognized_at = -1;
-		for (j = G_N_ELEMENTS (arguments) - 1; j >= 0; j--) {
-			if ((arguments[j].short_name && strncmp (argv[i], arguments[j].short_name, short_arglen[j]) == 0) ||
-			    (arguments[j].long_name && strncmp (argv[i], arguments[j].long_name, long_arglen[j]) == 0)) {
-				recognized_at = j;
-				
-				if (arguments[j].local == local)
-					break;
-			}
-		}
-		
-		if (recognized_at < 0) {
-		  	/* Argument not found in list.  Either its "local" setting mismatched
-			 * the one passed to us, or we got an invalid argument.  Check for the
-			 * latter only if we're doing local; otherwise just ignore the argument.
-			 */
-			if (local) {
-				g_printerr (_("Didn't understand argument '%s'\n"), argv[i]);
-				exit (1);
-			}
-			else {
-				continue;
-			}
-		}
-		else {
-			j = recognized_at;
-		}
-		
-		/* (from this point on, we know our argument was recognized) */
-		
-		longform = (strncmp (argv[i], "--", 2) == 0);
-		
-		if (arguments[j].local != local) {
-			/* We ignore this argument, but if it is in short form and expects a
-			 * companion argument, make sure we skip the companion argument on our
-			 * next iteration.
-			 */
-			if (arguments[j].type != XPAD_ARG_TYPE_NONE)
-				i++;
-			
-			/* Don't accept this argument, but don't complain either. */
-			continue;
-		}
-
-		if (arguments[j].type != XPAD_ARG_TYPE_NONE) {
-			long int int_arg;
-			char *companion, *endptr;
-			
-			if (argv[i][longform ? long_arglen[j] : short_arglen[j]] == '=') {
-				companion = &argv[i][longform ? long_arglen[j] : short_arglen[j] + 1];
-			}
-			else {
-				i++;
-				if (i >= argc)
-				  	missing_companion_arg (longform ? arguments[j].long_name : arguments[j].short_name);
-				
-				companion = argv[i];
-			}
-			
-			if (!*companion)
-				missing_companion_arg (longform ? arguments[j].long_name : arguments[j].short_name);
-			
-			if (arguments[j].type == XPAD_ARG_TYPE_INT) {
-				int_arg = strtol (companion, &endptr, 10);
-				
-				if (*endptr) {
-					g_printerr (_("Invalid number: '%s'\n"), companion);
-					
-					if (local)
-						exit(1);
-				}
-				else {
-					arguments[j].callbacks.func_arg (&int_arg);
-				}
-			}
-			else { /* string */
-				arguments[j].callbacks.func_arg (companion);
-			}
-		}
-		else {
-			arguments[j].callbacks.func ();
-		}
-		rv++;
-	}
+	g_option_context_free (context);
 	
-	return rv;
+	return(option_new || option_quit || option_smid);
 }
