@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2001-2004 Michael Terry
+Copyright (c) 2001-2005 Michael Terry
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "../config.h"
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 #include <string.h>
 #include "fio.h"
 #include "help.h"
@@ -39,6 +40,7 @@ struct XpadPadPrivate
 {
 	/* saved values */
 	gint x, y, width, height;
+	gboolean location_valid;
 	gchar *infoname;
 	gchar *contentname;
 	gboolean sticky;
@@ -55,13 +57,18 @@ struct XpadPadPrivate
 	/* properties window */
 	GtkWidget *properties;
 	
+	/* menus */
+	GtkWidget *menu;
+	GtkWidget *highlight_menu;
+	
 	XpadPadGroup *group;
 };
 
-/*enum
+enum
 {
+	CLOSED,
 	LAST_SIGNAL
-};*/
+};
 
 enum
 {
@@ -74,8 +81,11 @@ static void load_info (XpadPad *pad);
 static void save_info (XpadPad *pad);
 static void load_content (XpadPad *pad);
 static void save_content (XpadPad *pad);
+static GtkWidget *menu_get_popup_highlight (XpadPad *pad, GtkAccelGroup *accel_group);
+static GtkWidget *menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group);
 static void xpad_pad_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void xpad_pad_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void xpad_pad_dispose (GObject *object);
 static void xpad_pad_finalize (GObject *object);
 static void xpad_pad_show (XpadPad *pad);
 static gboolean xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event);
@@ -83,6 +93,7 @@ static gboolean xpad_pad_toolbar_size_allocate (XpadPad *pad, GtkAllocation *eve
 static gboolean xpad_pad_window_state_event (XpadPad *pad, GdkEventWindowState *event);
 static gboolean xpad_pad_delete_event (XpadPad *pad, GdkEvent *event);
 static gboolean xpad_pad_popup_menu (XpadPad *pad);
+static void xpad_pad_popup_deactivate (GtkWidget *menu, XpadPad *pad);
 static gboolean xpad_pad_button_press_event (XpadPad *pad, GdkEventButton *event);
 static gboolean xpad_pad_text_view_button_press_event (GtkWidget *text_view, GdkEventButton *event, XpadPad *pad);
 static void xpad_pad_text_changed (XpadPad *pad, GtkTextBuffer *buffer);
@@ -100,7 +111,6 @@ static void xpad_pad_delete (XpadPad *pad);
 static void xpad_pad_open_properties (XpadPad *pad);
 static void xpad_pad_open_preferences (XpadPad *pad);
 static void xpad_pad_quit (XpadPad *pad);
-static void xpad_pad_toggle_sticky (XpadPad *pad);
 static void xpad_pad_sync_title (XpadPad *pad);
 static void xpad_pad_set_group (XpadPad *pad, XpadPadGroup *group);
 static gboolean xpad_pad_leave_notify_event (GtkWidget *pad, GdkEventCrossing *event);
@@ -109,7 +119,7 @@ static void xpad_pad_toolbar_popup (GtkWidget *toolbar, GtkMenu *menu, XpadPad *
 static void xpad_pad_toolbar_popdown (GtkWidget *toolbar, GtkMenu *menu, XpadPad *pad);
 static XpadPadGroup *xpad_pad_get_group (XpadPad *pad);
 
-/*static guint signals[LAST_SIGNAL] = { 0 };*/
+static guint signals[LAST_SIGNAL] = { 0 };
 
 GtkWidget *
 xpad_pad_new (XpadPadGroup *group)
@@ -134,10 +144,21 @@ xpad_pad_class_init (XpadPadClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	
+	gobject_class->dispose = xpad_pad_dispose;
 	gobject_class->finalize = xpad_pad_finalize;
 	gobject_class->set_property = xpad_pad_set_property;
 	gobject_class->get_property = xpad_pad_get_property;
 	
+	signals[CLOSED] =
+		g_signal_new ("closed",
+		              G_OBJECT_CLASS_TYPE (gobject_class),
+		              G_SIGNAL_RUN_FIRST,
+		              G_STRUCT_OFFSET (XpadPadClass, closed),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
+	
 	/* Properties */
 	
 	g_object_class_install_property (gobject_class,
@@ -154,11 +175,13 @@ static void
 xpad_pad_init (XpadPad *pad)
 {
 	GtkWidget *vbox;
+	GtkAccelGroup *accel_group;
 	
 	pad->priv = XPAD_PAD_GET_PRIVATE (pad);
 	
 	pad->priv->x = 0;
 	pad->priv->y = 0;
+	pad->priv->location_valid = FALSE;
 	pad->priv->width = xpad_settings_get_width (xpad_settings ());
 	pad->priv->height = xpad_settings_get_height (xpad_settings ());
 	pad->priv->infoname = NULL;
@@ -189,6 +212,12 @@ xpad_pad_init (XpadPad *pad)
 	pad->priv->toolbar = GTK_WIDGET (g_object_new (XPAD_TYPE_TOOLBAR,
 		NULL));
 	
+	accel_group = gtk_accel_group_new ();
+	gtk_window_add_accel_group (GTK_WINDOW (pad), accel_group);
+	g_object_unref (G_OBJECT (accel_group));
+	pad->priv->menu = menu_get_popup_no_highlight (pad, accel_group);
+	pad->priv->highlight_menu = menu_get_popup_highlight (pad, accel_group);
+	
 	vbox = GTK_WIDGET (g_object_new (GTK_TYPE_VBOX,
 		"homogeneous", FALSE,
 		"spacing", 0,
@@ -201,7 +230,6 @@ xpad_pad_init (XpadPad *pad)
 		"decorated", xpad_settings_get_has_decorations (xpad_settings ()),
 		"default-height", xpad_settings_get_height (xpad_settings ()),
 		"default-width", xpad_settings_get_width (xpad_settings ()),
-		"gravity", GDK_GRAVITY_STATIC,
 		"skip-pager-hint", TRUE,
 		"skip-taskbar-hint", TRUE,
 		"type", GTK_WINDOW_TOPLEVEL,
@@ -241,11 +269,13 @@ xpad_pad_init (XpadPad *pad)
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-properties", G_CALLBACK (xpad_pad_open_properties), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-preferences", G_CALLBACK (xpad_pad_open_preferences), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-quit", G_CALLBACK (xpad_pad_quit), pad);
-	g_signal_connect_swapped (pad->priv->toolbar, "activate-sticky", G_CALLBACK (xpad_pad_toggle_sticky), pad);
 	
 	g_signal_connect (pad->priv->toolbar, "popup", G_CALLBACK (xpad_pad_toolbar_popup), pad);
 	g_signal_connect (pad->priv->toolbar, "popdown", G_CALLBACK (xpad_pad_toolbar_popdown), pad);
 	
+	g_signal_connect (pad->priv->menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
+	g_signal_connect (pad->priv->highlight_menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
+	
 	if (xpad_settings_get_sticky (xpad_settings ()))
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
@@ -269,7 +299,8 @@ xpad_pad_show (XpadPad *pad)
 	   again here after being shown.  This may create a visual effect if 
 	   the wm did ignore us, but is better than being in the wrong
 	   place, I guess. */
-	gtk_window_move (GTK_WINDOW (pad), pad->priv->x, pad->priv->y);
+	if (pad->priv->location_valid)
+		gtk_window_move (GTK_WINDOW (pad), pad->priv->x, pad->priv->y);
 	
 /*	g_object_set (G_OBJECT (pad),
 		"gravity", GDK_GRAVITY_STATIC,
@@ -286,13 +317,24 @@ xpad_pad_show (XpadPad *pad)
 }
 
 static void
-xpad_pad_finalize (GObject *object)
+xpad_pad_dispose (GObject *object)
 {
 	XpadPad *pad = XPAD_PAD (object);
 	
 	if (pad->priv->properties)
 		gtk_widget_destroy (pad->priv->properties);
 	
+	gtk_widget_destroy (pad->priv->menu);
+	gtk_widget_destroy (pad->priv->highlight_menu);
+	
+	G_OBJECT_CLASS (xpad_pad_parent_class)->dispose (object);
+}
+
+static void
+xpad_pad_finalize (GObject *object)
+{
+	XpadPad *pad = XPAD_PAD (object);
+	
 	g_free (pad->priv->infoname);
 	g_free (pad->priv->contentname);
 	
@@ -476,6 +518,8 @@ xpad_pad_close (XpadPad *pad)
 		gtk_widget_destroy (pad->priv->properties);
 	
 	save_info (pad);
+	
+	g_signal_emit (pad, signals[CLOSED], 0);
 }
 
 static gboolean
@@ -552,7 +596,7 @@ pad_properties_destroyed (XpadPad *pad)
 	if (!pad->priv->properties)
 		return;
 	
-	g_signal_handlers_disconnect_by_func (pad, (void *) pad_properties_sync_title, NULL);
+	g_signal_handlers_disconnect_by_func (pad, (gpointer) pad_properties_sync_title, NULL);
 	pad->priv->properties = NULL;
 }
 
@@ -683,19 +727,6 @@ xpad_pad_quit (XpadPad *pad)
 	gtk_main_quit ();
 }
 
-static void
-xpad_pad_toggle_sticky (XpadPad *pad)
-{
-	if (pad->priv->sticky)
-	{
-		gtk_window_unstick (GTK_WINDOW (pad));
-	}
-	else
-	{
-		gtk_window_stick (GTK_WINDOW (pad));
-	}
-}
-
 static void
 xpad_pad_text_changed (XpadPad *pad, GtkTextBuffer *buffer)
 {
@@ -720,6 +751,7 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 	pad->priv->y = event->y;
 	pad->priv->width = event->width;
 	pad->priv->height = event->height;
+	pad->priv->location_valid = TRUE;
 	
 	save_info (pad);
 	
@@ -731,8 +763,6 @@ xpad_pad_window_state_event (XpadPad *pad, GdkEventWindowState *event)
 {
 	pad->priv->sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ? TRUE : FALSE;
 	
-	xpad_toolbar_set_sticky_active (XPAD_TOOLBAR (pad->priv->toolbar), pad->priv->sticky);
-	
 	if (event->changed_mask & GDK_WINDOW_STATE_STICKY)
 		save_info (pad);
 	
@@ -943,7 +973,7 @@ load_info (XpadPad *pad)
 	gboolean locked = FALSE, follow_font = TRUE, follow_color = TRUE;
 	gboolean hidden = FALSE;
 	GdkColor text = {0}, back = {0};
-	gchar *fontname = NULL;
+	gchar *fontname = NULL, *oldcontentprefix;
 	
 	if (!pad->priv->infoname)
 		return;
@@ -969,6 +999,7 @@ load_info (XpadPad *pad)
 		NULL))
 		return;
 	
+	pad->priv->location_valid = TRUE;
 	gtk_window_set_default_size (GTK_WINDOW (pad), pad->priv->width, pad->priv->height);
 	gtk_window_move (GTK_WINDOW (pad), pad->priv->x, pad->priv->y);
 	
@@ -1000,6 +1031,20 @@ load_info (XpadPad *pad)
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
 	
+	/* Special check for contentname being absolute.  A while back,
+	   xpad had absolute pathnames, pointing to ~/.xpad/content-*.
+	   Now, files are kept in ~/.config/xpad, so using old config
+	   files with a new xpad will break pads.  We check to see if
+	   contentname is old pointer and then make it relative. */
+	oldcontentprefix = g_build_filename (g_get_home_dir (), ".xpad", "content-", NULL);
+	if (g_str_has_prefix (pad->priv->contentname, oldcontentprefix))
+	{
+		gchar *oldcontent = pad->priv->contentname;
+		pad->priv->contentname = g_path_get_basename (oldcontent);
+		g_free (oldcontent);
+	}
+	g_free (oldcontentprefix);
+	
 	if (hidden)
 		gtk_widget_hide (GTK_WIDGET (pad));
 }
@@ -1058,25 +1103,43 @@ save_info (XpadPad *pad)
 static void
 menu_about (XpadPad *pad)
 {
-	GtkWidget *dialog;
-	gchar *text;
+	const gchar *artists[] = {"Michael Terry <mike@mterry.name>", NULL};
+	const gchar *authors[] = {"Michael Terry <mike@mterry.name>", "Jeroen Vermeulen <jtv@xs4all.nl>", NULL};
+	const gchar *comments = _("Sticky notes");
+	const gchar *copyright = "© 2001-2005 Michael Terry";
+	/* we use g_strdup_printf because C89 has size limits on static strings */
+	gchar *license = g_strdup_printf ("%s\n%s\n%s",
+"This program is free software; you can redistribute it and/or\n"
+"modify it under the terms of the GNU General Public License\n"
+"as published by the Free Software Foundation; either version 2\n"
+"of the License, or (at your option) any later version.\n"
+,
+"This program is distributed in the hope that it will be useful,\n"
+"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+"GNU General Public License for more details.\n"
+,
+"You should have received a copy of the GNU General Public License\n"
+"along with this program; if not, write to the Free Software\n"
+"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.");
+	/* Translators: please translate this as your own name and optionally email
+	   like so: "Your Name <your@email.com>" */
+	const gchar *translator_credits = _("translator-credits");
+	const gchar *website = "http://xpad.sourceforge.net/";
 	
-	text = g_strdup_printf (_("You are using xpad %s with GTK+ %i.%i.%i."),
-		VERSION, gtk_major_version, gtk_minor_version, gtk_micro_version);
+	gtk_show_about_dialog (GTK_WINDOW (pad),
+		"artists", artists,
+		"authors", authors,
+		"comments", comments,
+		"copyright", copyright,
+		"license", license,
+		"logo-icon-name", PACKAGE,
+		"translator-credits", translator_credits,
+		"version", VERSION,
+		"website", website,
+		NULL);
 	
-	dialog = xpad_app_alert_new (GTK_WINDOW (pad), GTK_STOCK_DIALOG_INFO,
-		text,
-		_("Visit http://xpad.sourceforge.net/ for more information about xpad."));
-	g_free (text);
-	
-	if (!dialog)
-		return;
-	
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_OK, 1, NULL);
-	
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	
-	gtk_widget_destroy (dialog);
+	g_free (license);
 }
 
 static void
@@ -1240,21 +1303,23 @@ menu_title_compare (GtkWindow *a, GtkWindow *b)
 	return rv;
 }
 
-#define MENU_ADD(mnemonic, image, callback) {\
+#define MENU_ADD(mnemonic, image, key, mask, callback) {\
 	item = gtk_image_menu_item_new_with_mnemonic (mnemonic);\
 	if (image) {\
 		GtkWidget *imgwidget = gtk_image_new_from_stock (image, GTK_ICON_SIZE_MENU);\
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), imgwidget);\
 	}\
 	g_signal_connect_swapped (item, "activate", G_CALLBACK (callback), pad);\
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1); i++;\
+	if (key)\
+		gtk_widget_add_accelerator(item, "activate", accel_group, key, mask, GTK_ACCEL_VISIBLE);\
+	gtk_container_add (GTK_CONTAINER (menu), item);\
 	gtk_widget_show (item);\
 	}
 
 #define MENU_ADD_STOCK(stock, callback) {\
-	item = gtk_image_menu_item_new_from_stock (stock, NULL);\
+	item = gtk_image_menu_item_new_from_stock (stock, accel_group);\
 	g_signal_connect_swapped (item, "activate", G_CALLBACK (callback), pad);\
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1); i++;\
+	gtk_container_add (GTK_CONTAINER (menu), item);\
 	gtk_widget_show (item);\
 	}
 
@@ -1262,28 +1327,26 @@ menu_title_compare (GtkWindow *a, GtkWindow *b)
 	item = gtk_check_menu_item_new_with_mnemonic (mnemonic);\
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), active);\
 	g_signal_connect_swapped (item, "toggled", G_CALLBACK (callback), pad);\
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1); i++;\
+	gtk_container_add (GTK_CONTAINER (menu), item);\
 	gtk_widget_show (item);\
 	}
 
 #define MENU_ADD_SEP() {\
 	item = gtk_separator_menu_item_new ();\
-	gtk_menu_attach (GTK_MENU (menu), item, 0, 1, i, i + 1); i++;\
+	gtk_container_add (GTK_CONTAINER (menu), item);\
 	gtk_widget_show (item);\
 	}
 
 static GtkWidget *
-menu_get_popup_no_highlight (XpadPad *pad)
+menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 {
 	GtkWidget *uppermenu, *menu, *item;
-	GtkClipboard *clipboard;
-	gint i = 0;
 	
 	uppermenu = gtk_menu_new ();
-	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_menu_set_accel_group (GTK_MENU (uppermenu), accel_group);
 	
 	item = gtk_menu_item_new_with_mnemonic (_("_Pad"));
-	gtk_menu_attach (GTK_MENU (uppermenu), item, 0, 1, i, i + 1); i++;
+	gtk_container_add (GTK_CONTAINER (uppermenu), item);
 	gtk_widget_show (item);
 	
 	menu = gtk_menu_new ();
@@ -1291,7 +1354,7 @@ menu_get_popup_no_highlight (XpadPad *pad)
 	
 	MENU_ADD_STOCK (GTK_STOCK_NEW, xpad_pad_spawn);
 	MENU_ADD_SEP ();
-	MENU_ADD_CHECK (_("_Sticky"), pad->priv->sticky, menu_sticky);
+	MENU_ADD_CHECK (_("Show on _All Workspaces"), pad->priv->sticky, menu_sticky);
 	MENU_ADD_STOCK (GTK_STOCK_PROPERTIES, xpad_pad_open_properties);
 	MENU_ADD_SEP ();
 	MENU_ADD_STOCK (GTK_STOCK_CLOSE, xpad_pad_close);
@@ -1299,20 +1362,20 @@ menu_get_popup_no_highlight (XpadPad *pad)
 	
 	
 	item = gtk_menu_item_new_with_mnemonic (_("_Edit"));
-	gtk_menu_attach (GTK_MENU (uppermenu), item, 0, 1, i, i + 1); i++;
+	gtk_container_add (GTK_CONTAINER (uppermenu), item);
 	gtk_widget_show (item);
 	
 	menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 	
 	MENU_ADD_STOCK (GTK_STOCK_PASTE, menu_paste);
-	gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
+	g_object_set_data (G_OBJECT (uppermenu), "paste", item);
 	MENU_ADD_SEP ();
 	MENU_ADD_STOCK (GTK_STOCK_PREFERENCES, xpad_pad_open_preferences);
 	
 	
 	item = gtk_menu_item_new_with_mnemonic (_("_View"));
-	gtk_menu_attach (GTK_MENU (uppermenu), item, 0, 1, i, i + 1); i++;
+	gtk_container_add (GTK_CONTAINER (uppermenu), item);
 	gtk_widget_show (item);
 	
 	menu = gtk_menu_new ();
@@ -1326,25 +1389,72 @@ menu_get_popup_no_highlight (XpadPad *pad)
 	
 	
 	item = gtk_menu_item_new_with_mnemonic (_("_Notes"));
-	gtk_menu_attach (GTK_MENU (uppermenu), item, 0, 1, i, i + 1); i++;
+	gtk_container_add (GTK_CONTAINER (uppermenu), item);
+	gtk_widget_show (item);
+	
+	menu = gtk_menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
+	g_object_set_data (G_OBJECT (uppermenu), "notes-menu", menu);
+	
+	MENU_ADD (_("_Show All"), NULL, 0, 0, menu_show_all);
+	MENU_ADD (_("_Close All"), GTK_STOCK_QUIT, GDK_q, GDK_CONTROL_MASK, xpad_pad_quit);
+	
+	/* The rest of the notes menu will get set up in the prep function below */
+	
+	item = gtk_menu_item_new_with_mnemonic (_("_Help"));
+	gtk_container_add (GTK_CONTAINER (uppermenu), item);
 	gtk_widget_show (item);
 	
 	menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 	
-	MENU_ADD (_("_Show All"), NULL, menu_show_all);
-	MENU_ADD (_("_Close All"), GTK_STOCK_QUIT, xpad_pad_quit);
-	MENU_ADD_SEP ();
+	MENU_ADD (_("_Contents"), GTK_STOCK_HELP, GDK_F1, 0, show_help);
+	MENU_ADD (_("_About"), GTK_STOCK_ABOUT, 0, 0, menu_about);
 	
-	if (pad->priv->group)
+	return uppermenu;
+}
+
+static void
+menu_prep_popup_no_highlight (XpadPad *current_pad, GtkWidget *uppermenu)
+{
+	GtkWidget *menu, *item;
+	GtkClipboard *clipboard;
+	
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	
+	item = g_object_get_data (G_OBJECT (uppermenu), "paste");
+	if (item)
+		gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
+	
+	menu = g_object_get_data (G_OBJECT (uppermenu), "notes-menu");
+	if (menu)
+	{
+		gint n = 1;
+		gchar *key;
+		
+		/* Remove old menu */
+		item = g_object_get_data (G_OBJECT (menu), "notes-sep");
+		while (item)
+		{
+			gtk_container_remove (GTK_CONTAINER (menu), item);
+			key = g_strdup_printf ("notes-%i", n++);
+			item = g_object_get_data (G_OBJECT (menu), key);
+			g_free (key);
+		}
+	}
+	if (menu && current_pad->priv->group)
 	{
 		GSList *pads, *l;
 		gint n;
+		GtkAccelGroup *accel_group = gtk_menu_get_accel_group (GTK_MENU (uppermenu));
+		
+		MENU_ADD_SEP ();
+		g_object_set_data (G_OBJECT (menu), "notes-sep", item);
 		
 		/**
 		 * Order pads according to title.
 		 */
-		pads = xpad_pad_group_get_pads (pad->priv->group);
+		pads = xpad_pad_group_get_pads (current_pad->priv->group);
 		
 		pads = g_slist_sort (pads, (GCompareFunc) menu_title_compare);
 		
@@ -1355,8 +1465,11 @@ menu_get_popup_no_highlight (XpadPad *pad)
 		{
 			gchar *title;
 			gchar *tmp_title;
+			gchar *key;
+			GtkWidget *pad = GTK_WIDGET (l->data);
 			
-			tmp_title = g_strdup (gtk_window_get_title (GTK_WINDOW (l->data)));
+			key = g_strdup_printf ("notes-%i", n);
+			tmp_title = g_strdup (gtk_window_get_title (GTK_WINDOW (pad)));
 			str_replace_tokens (&tmp_title, '_', "__");
 			if (n < 10)
 				title = g_strdup_printf ("_%i. %s", n, tmp_title);
@@ -1364,44 +1477,29 @@ menu_get_popup_no_highlight (XpadPad *pad)
 				title = g_strdup_printf ("%i. %s", n, tmp_title);
 			g_free (tmp_title);
 			
-			MENU_ADD (title, NULL, menu_show);
+			MENU_ADD (title, NULL, 0, 0, menu_show);
+			g_object_set_data (G_OBJECT (menu), key, item);
 			
 			g_free (title);
+			g_free (key);
 		}
 		g_slist_free (pads);
 	}
-	
-	
-	item = gtk_menu_item_new_with_mnemonic (_("_Help"));
-	gtk_menu_attach (GTK_MENU (uppermenu), item, 0, 1, i, i + 1); i++;
-	gtk_widget_show (item);
-	
-	menu = gtk_menu_new ();
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-	
-	MENU_ADD (_("_Contents"), GTK_STOCK_HELP, show_help);
-	MENU_ADD (_("_About"), GTK_STOCK_DIALOG_INFO, menu_about);
-	
-	return uppermenu;
 }
 
 static GtkWidget *
-menu_get_popup_highlight (XpadPad *pad)
+menu_get_popup_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 {
 	GtkWidget *menu, *item;
-	GtkClipboard *clipboard;
-	gint i = 0;
 	
 	menu = gtk_menu_new ();
-	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_menu_set_accel_group (GTK_MENU (menu), accel_group);
 	
 	MENU_ADD_STOCK (GTK_STOCK_CUT, menu_cut);
 	MENU_ADD_STOCK (GTK_STOCK_COPY, menu_copy);
 	MENU_ADD_STOCK (GTK_STOCK_PASTE, menu_paste);
-	gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
-	
+	g_object_set_data (G_OBJECT (menu), "paste", item);
 	MENU_ADD_SEP ();
-	
 	MENU_ADD_STOCK (GTK_STOCK_BOLD, menu_bold);
 	MENU_ADD_STOCK (GTK_STOCK_ITALIC, menu_italic);
 	MENU_ADD_STOCK (GTK_STOCK_UNDERLINE, menu_underline);
@@ -1410,6 +1508,19 @@ menu_get_popup_highlight (XpadPad *pad)
 	return menu;
 }
 
+static void
+menu_prep_popup_highlight (XpadPad *pad, GtkWidget *menu)
+{
+	GtkWidget *item;
+	GtkClipboard *clipboard;
+	
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	
+	item = g_object_get_data (G_OBJECT (menu), "paste");
+	if (item)
+		gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
+}
+
 static void
 menu_popup (GtkWidget *menu, XpadPad *pad)
 {
@@ -1463,15 +1574,20 @@ xpad_pad_popup (XpadPad *pad, GdkEventButton *event)
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (pad->priv->textview));
 	
 	if (gtk_text_buffer_get_selection_bounds (buffer, NULL, NULL))
-		menu = menu_get_popup_highlight (pad);
+	{
+		menu = pad->priv->highlight_menu;
+		menu_prep_popup_highlight (pad, menu);
+	}
 	else
-		menu = menu_get_popup_no_highlight (pad);
+	{
+		menu = pad->priv->menu;
+		menu_prep_popup_no_highlight (pad, menu);
+	}
 	
 	if (!menu)
 		return;
 	
 	menu_popup (GTK_WIDGET (menu), pad);
-	g_signal_connect (menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
 	
 	if (event)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
